﻿--[[
	Copyright (c) 2012, Lars "Goldpaw" Norberg
	
	Web: http://www.friendlydruid.com
	Contact: goldpaw@friendlydruid.com
	
	All rights reserved
]]--

local addon, ns = ...
local module = LibStub("gCore-3.0"):NewModule("GUIS-gUI: Merchant")

-- Lua API
local min, max = math.min, math.max
local print = print
local select = select

-- WoW API
local BuyMerchantItem = BuyMerchantItem
local CanMerchantRepair = CanMerchantRepair
local GetGuildBankMoney = GetGuildBankMoney
local GetGuildBankWithdrawMoney = GetGuildBankWithdrawMoney
local GetMoney = GetMoney
local GetMerchantItemLink = GetMerchantItemLink
local GetMerchantItemMaxStack = GetMerchantItemMaxStack
local GetContainerItemID = GetContainerItemID
local GetContainerItemInfo = GetContainerItemInfo
local GetContainerNumSlots = GetContainerNumSlots
local GetItemInfo = GetItemInfo
local GetRepairAllCost = GetRepairAllCost
local IsInGuild = IsInGuild
local RepairAllItems = RepairAllItems
local UseContainerItem = UseContainerItem

-- GUIS API
local M = LibStub("gMedia-3.0")
local L = LibStub("gLocale-1.0"):GetLocale(addon)
local C = LibStub("gDB-1.0"):GetDataBase("GUIS-gUI: Colors")
local F = LibStub("gDB-1.0"):GetDataBase("GUIS-gUI: Functions")
local CreateChatCommand = function(...) module:CreateChatCommand(...) end

local defaults = {
	autorepair 		= true;
	autosell 		= true;
	guildrepair 	= true;
	detailedreport 	= false;
}

local menuTable = {
	{
		type = "group";
		name = module:GetName();
		order = 1;
		virtual = true;
		children = {
			{
				type = "widget";
				element = "Title";
				order = 1;
				msg = F.getName(module:GetName());
			};
			{
				type = "group";
				order = 5;
				virtual = true;
				children = {
				};
			};
		};
	};
}

module.MERCHANT_SHOW = function(self)
	local gain, sold 		= 0, 0
	local repaired 			= false
	local useGuildFunds 	= (IsInGuild()) and GUIS_DB.merchant.guildrepair or false
	local usedGuildFunds 	= false
	local yourGuildFunds 	= min((GetGuildBankWithdrawMoney() ~= -1) and GetGuildBankWithdrawMoney() or GetGuildBankMoney(), GetGuildBankMoney())
	local repairCost 		= (select(1, GetRepairAllCost())) or 0

	local itemID, count, link, rarity, price, stack
	if (GUIS_DB.merchant.autosell == true) then
		for bag = 0,4,1 do 
			for slot = 1, GetContainerNumSlots(bag), 1 do 
				itemID = GetContainerItemID(bag, slot)
				if itemID then
					count = (select(2, GetContainerItemInfo(bag, slot)))
					_, link, rarity, _, _, _, _, _, _, _, price = GetItemInfo(itemID)

					if rarity == 0 then
						stack = (price or 0) * (count or 1)
						sold = sold + stack
						
						if GUIS_DB.merchant.detailedreport then
							print(L["-%s|cFF00DDDDx%d|r %s"]:format(link, count, ("[money:%d]"):format(stack):tag()))
						end
						
						UseContainerItem(bag, slot)
					end
				end 
			end 
		end
		gain = gain + sold
	end

	if sold > 0 then 
		print(L["Earned %s"]:format(("[money:%d]"):format(sold):tag()))
	end

	if GUIS_DB.merchant.autorepair and CanMerchantRepair() and repairCost > 0 then

		if max(GetMoney(), yourGuildFunds) > repairCost then
		
			if (useGuildFunds and yourGuildFunds > repairCost) and MerchantGuildBankRepairButton:IsEnabled() and MerchantGuildBankRepairButton:IsShown() then
				RepairAllItems(1)
				usedGuildFunds = true
				repaired = true
				print(L["You repaired your items for %s using Guild Bank funds"]:format(("|cffff0000[money:%d]|r"):format(repairCost):tag()))
	
			elseif GetMoney() > repairCost then
				RepairAllItems() 
				repaired = true
				print(L["You repaired your items for %s"]:format(("|cffff0000[money:%d]|r"):format(repairCost):tag()))
				
				gain = gain - repairCost
			end
			
		else
			print(L["You haven't got enough available funds to repair!"])
		end
	end

	if gain > 0 then
		print(L["Your profit is %s"]:format(("[money:%d]"):format(gain):tag()))
		
	elseif gain < 0 then 
		print(L["Your expenses are %s"]:format(("|cFFFF0000[money:%d]|r"):format(-gain):tag()))
	end

end

-- update menu options
module.PostUpdateGUI = function(self)
	LibStub("gCore-3.0"):GetModule("GUIS-gUI: OptionsMenu"):Refresh(F.getName(self:GetName()))
end

module.RestoreDefaults = function(self)
	GUIS_DB["merchant"] = DuplicateTable(defaults)
end

module.Init = function(self)
	GUIS_DB["merchant"] = GUIS_DB["merchant"] or {}
	GUIS_DB["merchant"] = ValidateTable(GUIS_DB["merchant"], defaults)
end

module.OnInit = function(self)
	if F.kill(self:GetName()) then 
		self:Kill()
		return 
	end
	
	-- pre-hook the modified click handler
	local OrigMerchantItemButton_OnModifiedClick = MerchantItemButton_OnModifiedClick
	MerchantItemButton_OnModifiedClick = function(self, ...)
		if (IsAltKeyDown()) then
			local ID = self:GetID()
			
			if (ID) then
				local max = select(8, GetItemInfo(GetMerchantItemLink(ID)))
				if (max and max > 1) then
					BuyMerchantItem(ID, GetMerchantItemMaxStack(ID))
				end
			end
		end
		
		OrigMerchantItemButton_OnModifiedClick(self, ...)
	end
	
	-- set the tooltip to reflect the added functionality
	ITEM_VENDOR_STACK_BUY = ITEM_VENDOR_STACK_BUY .. "|n" .. L["<Alt-Click to buy the maximum amount>"]
	
	-- create the options menu
	do
		local restoreDefaults = function()
			if (InCombatLockdown()) then 
				print(L["Can not apply default settings while engaged in combat."])
				return
			end
			
			-- restore all frame positions
			self:RestoreDefaults()
			
			-- request a restart if needed
			F.RestartIfScheduled()
		end
		
		local cancelChanges = function()
		end
		
		local applySettings = function()
		end
		
		local applyChanges = function()
		end
		
		local gOM = LibStub("gCore-3.0"):GetModule("GUIS-gUI: OptionsMenu")
		gOM:RegisterWithBlizzard(gOM:New(menuTable), F.getName(self:GetName()), 
 			"default", restoreDefaults, 
			"cancel", cancelChanges, 
			"okay", applyChanges)
	end

	CreateChatCommand(function() 
		GUIS_DB["merchant"].autorepair = true 
		print(L["Your items will now be automatically repaired when visiting a vendor with repair capabilities"])
	end, "enableautorepair")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].autorepair = false 
		print(L["Your items will no longer be automatically repaired"])
	end, "disableautorepair")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].autosell = true 
		print(L["Poor Quality items will now be automatically sold when visiting a vendor"])
	end, "enableautosell")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].autosell = false 
		print(L["Poor Quality items will no longer be automatically sold"])
	end, "disableautosell")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].guildrepair = true 
		print(L["Your items will now be repaired using the Guild Bank if the options and the funds are available"])
	end, "enableguildrepair")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].guildrepair = false 
		print(L["Your items will now be repaired using your own funds"])
	end, "disableguildrepair")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].detailedreport = true 
		print(L["Detailed reports for autoselling of Poor Quality items turned on"])
	end, "enabledetailedreport")
	
	CreateChatCommand(function() 
		GUIS_DB["merchant"].detailedreport = false 
		print(L["Detailed reports for autoselling of Poor Quality items turned off, only summary will be shown"])
	end, "disabledetailedreport")

	self:RegisterEvent("MERCHANT_SHOW")
end

